#include "../include/SerialUSBHost.h"


static SemaphoreHandle_t device_disconnected_sem;//XXX


static cdc_acm_host_device_config_t dev_config = {
        .connection_timeout_ms = 5000, // 5 seconds, enough time to plug the device in or experiment with timeout
        .out_buffer_size = 512,
        .in_buffer_size = 512,
        .event_cb = SerialUSBHost::handle_event,
        .data_cb = SerialUSBHost::handle_rx,
        .user_arg = NULL,
    };

static cdc_acm_line_coding_t line_coding;


static const char *TAG = "VCP example";

bool SerialUSBHost::handle_rx(const uint8_t *data, size_t data_len, void *arg)
{
    //ESP_LOGI("USB_HANDLE", "Recivend data !!");
    printf("%.*s", data_len, data); //BUG remouve debug
    return true;
}

void SerialUSBHost::handle_event(const cdc_acm_host_dev_event_data_t *event, void *user_ctx)
{
    static const char *TAG = "handle_event";
    switch (event->type) {
        case CDC_ACM_HOST_ERROR:
            ESP_LOGE(TAG, "CDC-ACM error has occurred, err_no = %d", event->data.error);
            break;
        case CDC_ACM_HOST_DEVICE_DISCONNECTED:
            ESP_LOGI(TAG, "Device suddenly disconnected");
            xSemaphoreGive(device_disconnected_sem);
            break;
        case CDC_ACM_HOST_SERIAL_STATE:
            ESP_LOGI(TAG, "Serial state notif 0x%04X", event->data.serial_state.val);
            break;
        case CDC_ACM_HOST_NETWORK_CONNECTION:
        default: break;
    }
}

void SerialUSBHost::usb_lib_task(void* arg) {
    static const char *TAG = "SerialUSBHost_usb_lib_task";
    while (1) {
        // Start handling system events
        uint32_t event_flags;
        usb_host_lib_handle_events(portMAX_DELAY, &event_flags);
        if (event_flags & USB_HOST_LIB_EVENT_FLAGS_NO_CLIENTS) {
            ESP_ERROR_CHECK(usb_host_device_free_all());
        }
        if (event_flags & USB_HOST_LIB_EVENT_FLAGS_ALL_FREE) {
            ESP_LOGI(TAG, "USB: All devices freed");
            // Continue handling USB events to allow device reconnection
        }
    }
}

void SerialUSBHost::init(uint32_t baudrate, uint8_t stop_bits, uint8_t parity, uint8_t dataBits) {
    static const char *TAG = "SerialUSBHost_constructor";
    device_disconnected_sem = xSemaphoreCreateBinary();
    assert(device_disconnected_sem);

    // Install USB Host driver. Should only be called once in entire application
    ESP_LOGI(TAG, "Installing USB Host");
    const usb_host_config_t host_config = {
        .skip_phy_setup = false,
        .intr_flags = ESP_INTR_FLAG_LEVEL1,
        .enum_filter_cb = nullptr,
    };
    ESP_ERROR_CHECK(usb_host_install(&host_config));

    // Create a task that will handle USB library events
    BaseType_t task_created = xTaskCreate(SerialUSBHost::usb_lib_task, "usb_lib", 4096, NULL, 10, NULL);
    assert(task_created == pdTRUE);

    ESP_LOGI(TAG, "Installing CDC-ACM driver");
    ESP_ERROR_CHECK(cdc_acm_host_install(NULL));

    // Register VCP drivers to VCP service
    esp_usb::VCP::register_driver<esp_usb::FT23x>();
    esp_usb::VCP::register_driver<esp_usb::CP210x>();
    esp_usb::VCP::register_driver<esp_usb::CH34x>();

    line_coding.dwDTERate = baudrate;
    line_coding.bCharFormat = stop_bits;
    line_coding.bParityType = parity;
    line_coding.bDataBits = dataBits;

    BaseType_t mainTask = xTaskCreate(SerialUSBHost::mainTask, "main usb task", 4096, NULL, 5, NULL);
    assert(mainTask == pdTRUE);
}

void SerialUSBHost::mainTask(void* arg) {
    while (1)
    {
        // You don't need to know the device's VID and PID. Just plug in any device and the VCP service will load correct (already registered) driver for the device
        ESP_LOGI(TAG, "Opening any VCP device...");
        auto vcp = std::unique_ptr<CdcAcmDevice>(esp_usb::VCP::open(&dev_config));

        if (vcp == nullptr) {
            ESP_LOGI(TAG, "Failed to open VCP device");
            return;
            // continue;
        }
        vTaskDelay(10);

        ESP_LOGI(TAG, "Setting up line coding");

        ESP_ERROR_CHECK(vcp->line_coding_set(&line_coding));

        /*
        Now the USB-to-UART converter is configured and receiving data.
        You can use standard CDC-ACM API to interact with it. E.g.

        ESP_ERROR_CHECK(vcp->set_control_line_state(false, true));
        ESP_ERROR_CHECK(vcp->tx_blocking((uint8_t *)"Test string", 12));
        */

        // Send some dummy data
        ESP_LOGI(TAG, "Sending data through CdcAcmDevice");
        uint8_t data[] = "test_string";
        ESP_ERROR_CHECK(vcp->tx_blocking(data, sizeof(data)));
        ESP_ERROR_CHECK(vcp->set_control_line_state(true, true));

        // We are done. Wait for device disconnection and start over
        ESP_LOGI(TAG, "Done. You can reconnect the VCP device to run again.");

        xSemaphoreTake(device_disconnected_sem, portMAX_DELAY);
    }
}


//TODO: faire une fontion pour savoir si un usb est connecter
